/* 
 * File:   inspect-file.cpp
 * Author: michael
 *
 * Created on 25. Oktober 2015, 01:26
 */

#include <fstream>
#include <cstdlib>

#include <rsc/misc/SignalWaiter.h>

#include <rsc/logging/LoggerFactory.h>
#include <rsbag/Bag.h>

#include <rst/classification/ClassificationResult.pb.h>
#include <rst/tracking/TrackedPostures3DFloat.pb.h>
#include <rst/hri/GroupHypothesis.pb.h>
#include <rst/vision/HeadObjects.pb.h>
#include <rst/hri/PersonHypotheses.pb.h>
#include <rst/audition/SoundChunk.pb.h>
#include <rst/geometry/Pose.pb.h>
#include <rst/kinematics/ProprioceptionState.pb.h>
#include <rst/generic/MethodCall.pb.h>
#include <rst/generic/Value.pb.h>

#include <rsb/MetaData.h>

using namespace std;
using namespace boost;
using namespace rsbag;
using namespace rsc::misc;

/*
 * 
 */
int main(int argc, char** argv) {
    
    initSignalWaiter();
    
//    registerType<rst::classification::ClassificationResult>();
//    registerType<rst::generic::MethodCall>();
//    registerType<rst::generic::Value>();
//    registerType<rst::tracking::TrackedPostures3DFloat>();
//    registerType<rst::vision::HeadObjects>();
//    registerType<rst::hri::GroupHypothesis>();
    registerType<rst::hri::PersonHypotheses>();
//    registerType<rst::audition::SoundChunk>();
//    registerType<rst::geometry::Pose>();
//    registerType<rst::kinematics::ProprioceptionState>();
    
    set<string> channels;
    channels.insert("/home/wardrobe/cam1/person");
    channels.insert("/home/wardrobe/cam2/person");
    channels.insert("/home/kitchen/cam1/person");
    channels.insert("/home/kitchen/cam2/person");
    channels.insert("/home/living/cam1/person");
    channels.insert("/home/living/cam2/person");
    channels.insert("/home/living/cam3/person");
    channels.insert("/home/living/cam4/person");
    channels.insert("/home/sports/cam1/person");
    channels.insert("/home/sports/cam2/person");
    channels.insert("/home/sports/cam3/person");
    channels.insert("/home/sports/cam4/person");
    
    cout << "Opening " << argv[1] << endl;
    
    Bag funnybag(argv[1], channels, 60);
    
    ofstream out(argv[2]);
    
    cout << "File contains " << funnybag.size() << " events!" << endl;
    
    try {
        for(Bag::iterator it = funnybag.begin(); it != funnybag.end() && lastArrivedSignal() == NO_SIGNAL; it++) {
//            cout << "Event with type " << it->getScope() << ": " << it->getType() << endl;
            
            shared_ptr<rst::hri::PersonHypotheses> data = static_pointer_cast<rst::hri::PersonHypotheses>(it->getData());
            
            for(int i=0; i<data->persons_size(); i++) {
                float x = data->persons(i).body().shape().transformation().translation().x();
                float y = data->persons(i).body().shape().transformation().translation().y();
                float z = data->persons(i).body().shape().transformation().translation().z();
                
                int id = data->persons(i).tracking_info().id();
                
                out << it->getMetaData().getCreateTime() << ", " << it->getScope().toString() << ", " << id << ", " << x << ", " << y << ", " << z << ", " << data->persons(i).body().location().frame_id() << endl;
            }
        }
    } catch(...) {
        cout << "Error while receiving..." << endl;
    }
    
    cout << "Will exit..." << endl;
    
    if(lastArrivedSignal() != NO_SIGNAL) {
        return suggestedExitCode(lastArrivedSignal());
    }
    
    return 0;
}

