/* 
 * File:   Process.h
 * Author: michael
 *
 * Created on 24. Oktober 2015, 12:10
 */

#ifndef PROCESS_H
#define	PROCESS_H

#include <boost/format.hpp>
#include <boost/shared_ptr.hpp>

#include <rsc/logging/Logger.h>

#ifdef __linux__
#include <rsbag/Process_impl_linux.h>
#endif //__linux__

#ifdef _win32
#include <rsbag/Process_imp_windows.h>
#endif

namespace rsbag {

    class Process_impl;

    /**
     * A class representing a process. It is supposed to support linux as well
     * as windows, though it's implemented only for linux at the moment.
     */
    class Process {
    public:

        /**
         * Construct a new process with the given programm, arguments and 
         * environment. The process will not be started automatically.
         * Use start() to make it run.
         * 
         * @param program the programm to load
         * @param arguments the arguments to pass
         * @param environment the environment to give.
         */
        Process(std::string program,
                std::vector<std::string> arguments = std::vector<std::string>(),
                std::vector<std::string> environment = std::vector<std::string>());

        /**
         * Start the process.
         */
        void start();

        /**
         * Stop the process. Will send an interrupt to the process, then wait
         * for 5 seconds for it to finish. If this won't work force-end the
         * process and fetch its result.
         * 
         * @return the exit code of the process.
         */
        int stop(unsigned int wait_time = 5);

        virtual ~Process();
    private:
        Process_impl impl;

        rsc::logging::LoggerPtr logger;
    };

    typedef boost::shared_ptr<Process> ProcessPtr;

}
#endif	/* PROCESS_H */

