/* 
 * File:   Process_imp_linux.cpp
 * Author: michael
 * 
 * Created on 24. Oktober 2015, 12:11
 */

#ifdef __linux__

#include "Process_impl_linux.h"

using namespace std;
using namespace rsc::logging;
using namespace rsbag;


Process_impl::Process_impl(string program, vector<string> arguments, vector<string> environment) :
program(program), arguments(arguments), environment(environment), process(-1),
logger(Logger::getLogger("rsbag.Process")) {
}

void Process_impl::start() {
    RSCDEBUG(logger, "Forking...");
    process = fork();

    if (process == 0) {
        arguments.insert(arguments.begin(), program);
        char** c_arguments = getAsCharArrayArray(arguments);
        char** c_environment = getAsCharArrayArray(environment);
        
        if(setsid() == -1) {
            RSCFATAL(logger, boost::str(boost::format("Failed to setsid: errno %1%.") % errno));
        }

        int res = execve(program.c_str(), c_arguments, c_environment);

        if (res == -1) {
            RSCFATAL(logger, boost::str(boost::format("Failed to execve: errno %1%.") % errno));
        }
    }
}

bool Process_impl::is_alive() {
    RSCDEBUG(logger, "Checking if process is alive..");
    
    int status = 0;

    pid_t res = waitpid(process, &status, WNOHANG | WNOWAIT);

    if (res == -1) {
        return false;
    }

    return WIFEXITED(status);
}

void Process_impl::request_end() {
    RSCDEBUG(logger, "Sending SIGINT to child process...");

    if (kill(process, SIGINT) == -1) {
        RSCERROR(logger, "kill returned -1!");
    }
}

void Process_impl::force_end() {
    RSCDEBUG(logger, "Sending SIGKILL to child process...");

    if (kill(process, SIGKILL) == -1) {
        RSCERROR(logger, "kill returned -1!");
    }
}

bool Process_impl::is_exited(int result) {
    return WIFEXITED(result);
}

int Process_impl::wait_for_end(unsigned int timeout) {
    RSCDEBUG(logger, "Waiting for process end...");
    
    int status = 0;
    
    time_t start_time = time(NULL);
    time_t now;
    pid_t res;
    
    bool should_poll = false;

    // poll for ending of the process...
    do {
        sched_yield();
        now = time(NULL);
        
        should_poll = (timeout == 0) || (now - start_time < timeout);
    } while(is_alive() && should_poll);
    
    // check if exited
    if(!is_alive()) {
        // clear results
        RSCDEBUG(logger, "fetching process result...");
        res = waitpid(process, &status, 0);
    }

    return status;
}

char** Process_impl::getAsCharArrayArray(vector<string>& object) {
    char** result = new char*[object.size() + 1];

    for (int i = 0; i < object.size(); i++) {
        result[i] = new char[object[i].size() + 1];
        object[i].copy(result[i], object[i].size());
        result[i][object[i].size()] = 0;
    }

    result[object.size()] = NULL;

    return result;
}

Process_impl::~Process_impl() {
    if(is_alive()) {
        force_end();
        wait_for_end();
    }
}


#endif